@@ -18,12 +18,28 @@
 #include <stdlib.h>
 
 static void
-usage (const char* argv0)
+usage (FILE* stream, const char* argv0, int verbose)
 {
     fprintf (
-        stderr,
+        stream,
         "Usage: %s [-v|--verbose] [-a|--all-metadata] [-s|--strict] <filename> [<filename> ...]\n\n",
         argv0);
+
+    if (verbose)
+        fprintf (
+            stream,
+            "\n"
+            "Read exr files and print values of header attributes\n"
+            "\n"
+            "Options:\n"
+            "  -s, --strict        strict mode\n"
+            "  -a, --all-metadata  print all metadata\n"
+            "  -v, --verbose       verbose mode\n"
+            "  -h, --help          print this message\n"
+            "      --version       print version information\n"
+            "\n"
+            "Report bugs via https://github.com/AcademySoftwareFoundation/openexr/issues or email security@openexr.com\n"
+            "");
 }
 
 static void
@@ -136,7 +152,16 @@ main (int argc, const char* argv[])
         if (!strcmp (argv[a], "-h") || !strcmp (argv[a], "-?") ||
             !strcmp (argv[a], "--help"))
         {
-            usage (argv[0]);
+            usage (stdout, "exrinfo", 1);
+            return 0;
+        }
+        else if (!strcmp (argv[a], "--version"))
+        {
+            printf (
+                "exrinfo (OpenEXR) %s  https://openexr.com\n",
+                OPENEXR_VERSION_STRING);
+            printf ("Copyright (c) Contributors to the OpenEXR Project\n");
+            printf ("License BSD-3-Clause\n");
             return 0;
         }
         else if (!strcmp (argv[a], "-v") || !strcmp (argv[a], "--verbose"))
@@ -157,13 +182,10 @@ main (int argc, const char* argv[])
         }
         else if (argv[a][0] == '-')
         {
-            usage (argv[0]);
+            usage (stderr, argv[0], 0);
             return 1;
         }
-        else
-        {
-            rv += process_file (argv[a], verbose, allmeta, strict);
-        }
+        else { rv += process_file (argv[a], verbose, allmeta, strict); }
     }
 
     return rv;
