@@ -71,7 +71,7 @@ reorder_and_predict (void* scratch, const void* packed, uint64_t packedbytes)
     int8_t*       t2   = t1 + (packedbytes + 1) / 2;
     const int8_t* in   = packed;
     const int8_t* stop = in + packedbytes;
-    int d, p;
+    int           d, p;
 
     while (in < stop)
     {
@@ -79,9 +79,9 @@ reorder_and_predict (void* scratch, const void* packed, uint64_t packedbytes)
         if (in < stop) *(t2++) = *(in++);
     }
 
-    t1    = scratch;
-    stop  = t1 + packedbytes;
-    p = *(t1++);
+    t1   = scratch;
+    stop = t1 + packedbytes;
+    p    = *(t1++);
     while (t1 < stop)
     {
         d     = (int) (*t1) - p + (128 + 256);
@@ -140,8 +140,8 @@ internal_rle_decompress (
         {
             uint64_t count = (uint64_t) (-((int) *in++));
             ++unpackbytes;
-            if (unpackbytes + count > packsz) return EXR_ERR_CORRUPT_CHUNK;
-            if (outbytes + count > outsz) return EXR_ERR_CORRUPT_CHUNK;
+            if (unpackbytes + count > packsz) return 0;
+            if (outbytes + count > outsz) return 0;
 
             memcpy (dst, in, count);
             in += count;
@@ -152,11 +152,11 @@ internal_rle_decompress (
         else
         {
             uint64_t count = (uint64_t) (*in++);
-            if (unpackbytes + 2 > packsz) return EXR_ERR_CORRUPT_CHUNK;
+            if (unpackbytes + 2 > packsz) return 0;
             unpackbytes += 2;
 
             ++count;
-            if (outbytes + count > outsz) return EXR_ERR_CORRUPT_CHUNK;
+            if (outbytes + count > outsz) return 0;
 
             memset (dst, *(const uint8_t*) in, count);
             dst += count;
@@ -179,7 +179,7 @@ unpredict_and_reorder (void* out, void* scratch, uint64_t packedbytes)
     while (t1 < stop)
     {
         int d = (int) (t1[-1]) + (int) (t1[0]) - 128;
-        t1[0] = (int8_t)d;
+        t1[0] = (int8_t) d;
         ++t1;
     }
 
@@ -202,6 +202,13 @@ internal_exr_undo_rle (
 {
     exr_result_t rv;
     uint64_t     unpackb;
+
+    if (packsz == 0 || src == NULL)
+    {
+        decode->bytes_decompressed = 0;
+        return EXR_ERR_SUCCESS;
+    }
+
     rv = internal_decode_alloc_buffer (
         decode,
         EXR_TRANSCODE_BUFFER_SCRATCH1,
@@ -212,8 +219,12 @@ internal_exr_undo_rle (
 
     unpackb =
         internal_rle_decompress (decode->scratch_buffer_1, outsz, src, packsz);
-    if (unpackb != outsz) return EXR_ERR_CORRUPT_CHUNK;
+    if (unpackb != outsz)
+        return EXR_ERR_CORRUPT_CHUNK;
+
+    unpredict_and_reorder (out, decode->scratch_buffer_1, unpackb);
+
+    decode->bytes_decompressed = unpackb;
 
-    unpredict_and_reorder (out, decode->scratch_buffer_1, outsz);
     return EXR_ERR_SUCCESS;
 }
