@@ -19,7 +19,7 @@
 /**************************************/
 
 static exr_result_t
-save_attr_sz (struct _internal_exr_context* ctxt, size_t sz)
+save_attr_sz (exr_context_t ctxt, size_t sz)
 {
     int32_t isz;
 
@@ -36,7 +36,7 @@ save_attr_sz (struct _internal_exr_context* ctxt, size_t sz)
 /**************************************/
 
 static exr_result_t
-save_attr_32 (struct _internal_exr_context* ctxt, void* ptr, int n)
+save_attr_32 (exr_context_t ctxt, void* ptr, int n)
 {
     priv_from_native32 (ptr, n);
 
@@ -50,7 +50,7 @@ save_attr_32 (struct _internal_exr_context* ctxt, void* ptr, int n)
 /**************************************/
 
 static exr_result_t
-save_attr_64 (struct _internal_exr_context* ctxt, void* ptr, int n)
+save_attr_64 (exr_context_t ctxt, void* ptr, int n)
 {
     priv_from_native64 (ptr, n);
 
@@ -64,7 +64,7 @@ save_attr_64 (struct _internal_exr_context* ctxt, void* ptr, int n)
 /**************************************/
 
 static exr_result_t
-save_attr_uint8 (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_attr_uint8 (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
 
@@ -78,7 +78,7 @@ save_attr_uint8 (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_attr_float (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_attr_float (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     float        tmp = a->f;
@@ -91,7 +91,7 @@ save_attr_float (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_attr_int (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_attr_int (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     int32_t      tmp = a->i;
@@ -104,7 +104,7 @@ save_attr_int (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_attr_double (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_attr_double (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     double       tmp = a->d;
@@ -117,7 +117,7 @@ save_attr_double (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_box2i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_box2i (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t     rv;
     exr_attr_box2i_t tmp = *(a->box2i);
@@ -130,7 +130,7 @@ save_box2i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_box2f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_box2f (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t     rv;
     exr_attr_box2f_t tmp = *(a->box2f);
@@ -143,7 +143,33 @@ save_box2f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_chlist (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_bytes (exr_context_t ctxt, const exr_attribute_t* a)
+{
+    exr_result_t rv;
+    int32_t      sz    = 0;
+    exr_attr_bytes_t* b = a->bytes;
+
+    if (!b || !b->data || b->size < 0)
+    {
+        return ctxt->report_error (
+            ctxt,
+            EXR_ERR_INVALID_ARGUMENT,
+            "Invalid reference to bytes attribute to save.");
+    }
+
+    sz = (int32_t) b->size;
+
+    rv = save_attr_sz (ctxt, (uint64_t) sz);
+    if (rv == EXR_ERR_SUCCESS && sz > 0)
+        rv = ctxt->do_write (
+            ctxt, b->data, (uint64_t) sz, &(ctxt->output_file_offset));
+    return rv;
+}
+
+/**************************************/
+
+static exr_result_t
+save_chlist (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     size_t       attrsz = 0;
@@ -195,7 +221,7 @@ save_chlist (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
     if (rv == EXR_ERR_SUCCESS)
     {
         eol = 0;
-        rv = ctxt->do_write (
+        rv  = ctxt->do_write (
             ctxt, &eol, sizeof (uint8_t), &(ctxt->output_file_offset));
     }
     return rv;
@@ -204,8 +230,7 @@ save_chlist (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_chromaticities (
-    struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_chromaticities (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t              rv;
     exr_attr_chromaticities_t tmp = *(a->chromaticities);
@@ -218,7 +243,7 @@ save_chromaticities (
 /**************************************/
 
 static exr_result_t
-save_float_vector (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_float_vector (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
 
@@ -259,7 +284,7 @@ save_float_vector (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_keycode (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_keycode (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t       rv;
     exr_attr_keycode_t tmp = *(a->keycode);
@@ -272,7 +297,7 @@ save_keycode (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_m33f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_m33f (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t    rv;
     exr_attr_m33f_t tmp = *(a->m33f);
@@ -285,7 +310,7 @@ save_m33f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_m33d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_m33d (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t    rv;
     exr_attr_m33d_t tmp = *(a->m33d);
@@ -298,7 +323,7 @@ save_m33d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_m44f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_m44f (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t    rv;
     exr_attr_m44f_t tmp = *(a->m44f);
@@ -311,7 +336,7 @@ save_m44f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_m44d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_m44d (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t    rv;
     exr_attr_m44d_t tmp = *(a->m44d);
@@ -324,7 +349,7 @@ save_m44d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_preview (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_preview (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     uint32_t     sizes[2];
@@ -346,7 +371,7 @@ save_preview (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_rational (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_rational (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t        rv;
     exr_attr_rational_t tmp = *(a->rational);
@@ -359,7 +384,7 @@ save_rational (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_string (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_string (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t       rv;
     exr_attr_string_t* tmp = a->string;
@@ -377,8 +402,7 @@ save_string (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_string_vector (
-    struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_string_vector (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     size_t       attrsz = 0;
@@ -411,7 +435,7 @@ save_string_vector (
 /**************************************/
 
 static exr_result_t
-save_tiledesc (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_tiledesc (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     uint32_t     sizes[2];
@@ -434,7 +458,7 @@ save_tiledesc (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_timecode (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_timecode (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t        rv;
     exr_attr_timecode_t tmp = *(a->timecode);
@@ -447,7 +471,7 @@ save_timecode (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v2i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v2i (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v2i_t tmp = *(a->v2i);
@@ -460,7 +484,7 @@ save_v2i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v2f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v2f (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v2f_t tmp = *(a->v2f);
@@ -473,7 +497,7 @@ save_v2f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v2d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v2d (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v2d_t tmp = *(a->v2d);
@@ -486,7 +510,7 @@ save_v2d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v3i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v3i (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v3i_t tmp = *(a->v3i);
@@ -499,7 +523,7 @@ save_v3i (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v3f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v3f (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v3f_t tmp = *(a->v3f);
@@ -512,7 +536,7 @@ save_v3f (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_v3d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_v3d (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t   rv;
     exr_attr_v3d_t tmp = *(a->v3d);
@@ -525,7 +549,7 @@ save_v3d (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_opaque (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_opaque (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
     int32_t      sz    = 0;
@@ -545,7 +569,7 @@ save_opaque (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
 /**************************************/
 
 static exr_result_t
-save_attr (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
+save_attr (exr_context_t ctxt, const exr_attribute_t* a)
 {
     exr_result_t rv;
 
@@ -563,6 +587,7 @@ save_attr (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
     {
         case EXR_ATTR_BOX2I: rv = save_box2i (ctxt, a); break;
         case EXR_ATTR_BOX2F: rv = save_box2f (ctxt, a); break;
+        case EXR_ATTR_BYTES: rv = save_bytes (ctxt, a); break;
         case EXR_ATTR_CHLIST: rv = save_chlist (ctxt, a); break;
         case EXR_ATTR_CHROMATICITIES: rv = save_chromaticities (ctxt, a); break;
         case EXR_ATTR_COMPRESSION: rv = save_attr_uint8 (ctxt, a); break;
@@ -598,22 +623,78 @@ save_attr (struct _internal_exr_context* ctxt, const exr_attribute_t* a)
     return rv;
 }
 
+/**************************************/
+
+exr_result_t internal_exr_calc_header_version_flags (exr_const_context_t ctxt, uint32_t *flags)
+{
+    *flags = 2; // EXR_VERSION
+
+    if (ctxt->is_multipart) *flags |= EXR_MULTI_PART_FLAG;
+
+    if (ctxt->max_name_length > EXR_SHORTNAME_MAXLEN)
+    {
+        int longnamefound = 0;
+
+        for ( int p = 0; p < ctxt->num_parts; ++p )
+        {
+            exr_priv_part_t curpart = ctxt->parts[p];
+            for ( int a = 0; a < curpart->attributes.num_attributes; ++a )
+            {
+                exr_attribute_t *cura = curpart->attributes.entries[a];
+                if (cura->name_length > EXR_SHORTNAME_MAXLEN ||
+                    cura->type_name_length > EXR_SHORTNAME_MAXLEN)
+                {
+                    longnamefound = 1;
+                    break;
+                }
+
+                // the original C++ side assumes there was
+                // only one channel list (who would have multiple)
+                // but let's not make that same assertion and check names
+                // if we encounter any channel list named anything
+                if (cura->type == EXR_ATTR_CHLIST)
+                {
+                    const exr_attr_chlist_t* chlist = cura->chlist;
+                    int nc = chlist->num_channels;
+
+                    for ( int c = 0; c < nc; ++c )
+                    {
+                        const exr_attr_chlist_entry_t *ce = chlist->entries + c;
+                        if (ce->name.length > EXR_SHORTNAME_MAXLEN)
+                        {
+                            longnamefound = 1;
+                            break;
+                        }
+                    }
+                }
+            }
+
+            if (longnamefound)
+                break;
+        }
+
+        if (longnamefound)
+            *flags |= EXR_LONG_NAMES_FLAG;
+    }
+
+    if (ctxt->has_nonimage_data) *flags |= EXR_NON_IMAGE_FLAG;
+    if (ctxt->is_singlepart_tiled) *flags |= EXR_TILED_FLAG;
+
+    return EXR_ERR_SUCCESS;
+}
+
+
 /**************************************/
 
 exr_result_t
-internal_exr_write_header (struct _internal_exr_context* ctxt)
+internal_exr_write_header (exr_context_t ctxt)
 {
     exr_result_t rv;
     uint32_t     magic_and_version[2];
     uint32_t     flags;
     uint8_t      next_byte;
 
-    flags = 2; // EXR_VERSION
-    if (ctxt->is_multipart) flags |= EXR_MULTI_PART_FLAG;
-    if (ctxt->max_name_length > EXR_SHORTNAME_MAXLEN)
-        flags |= EXR_LONG_NAMES_FLAG;
-    if (ctxt->has_nonimage_data) flags |= EXR_NON_IMAGE_FLAG;
-    if (ctxt->is_singlepart_tiled) flags |= EXR_TILED_FLAG;
+    rv = internal_exr_calc_header_version_flags (ctxt, &flags);
 
     magic_and_version[0] = 20000630;
     magic_and_version[1] = flags;
@@ -629,13 +710,13 @@ internal_exr_write_header (struct _internal_exr_context* ctxt)
 
     for (int p = 0; rv == EXR_ERR_SUCCESS && p < ctxt->num_parts; ++p)
     {
-        struct _internal_exr_part* curp = ctxt->parts[p];
+        exr_priv_part_t curp = ctxt->parts[p];
         if (ctxt->legacy_header)
         {
             for (int a = 0; a < curp->attributes.num_attributes; ++a)
             {
-                exr_attribute_t *curattr = curp->attributes.sorted_entries[a];
-                if (0 == (flags & (EXR_MULTI_PART_FLAG |EXR_NON_IMAGE_FLAG)) &&
+                exr_attribute_t* curattr = curp->attributes.sorted_entries[a];
+                if (0 == (flags & (EXR_MULTI_PART_FLAG | EXR_NON_IMAGE_FLAG)) &&
                     1 == ctxt->num_parts)
                 {
                     if (0 == strcmp (curattr->name, "type") ||
